@@ -1,32 +1,42 @@
 #include "hb-benchmark.hh"
 
+#include <glib.h>
+
 #define SUBSET_FONT_BASE_PATH "test/subset/data/fonts/"
 
-struct test_input_t
+static hb_variation_t default_variations[] = {
+    hb_variation_t {HB_TAG ('w','g','h','t'), 500},
+    hb_variation_t {HB_TAG_NONE, 0}
+};
+
+static unsigned max_num_glyphs = 0;
+static const char *text = " ";
+
+static struct test_input_t
 {
-  bool is_variable;
+  hb_variation_t *variations;
   const char *font_path;
 } default_tests[] =
 {
-  {false, SUBSET_FONT_BASE_PATH "Roboto-Regular.ttf"},
-  {true , SUBSET_FONT_BASE_PATH "RobotoFlex-Variable.ttf"},
-  {false, SUBSET_FONT_BASE_PATH "SourceSansPro-Regular.otf"},
-  {true , SUBSET_FONT_BASE_PATH "AdobeVFPrototype.otf"},
-  {true , SUBSET_FONT_BASE_PATH "SourceSerifVariable-Roman.ttf"},
-  {false, SUBSET_FONT_BASE_PATH "Comfortaa-Regular-new.ttf"},
-  {false, SUBSET_FONT_BASE_PATH "NotoNastaliqUrdu-Regular.ttf"},
-  {false, SUBSET_FONT_BASE_PATH "NotoSerifMyanmar-Regular.otf"},
+  {nullptr,            SUBSET_FONT_BASE_PATH "Roboto-Regular.ttf"},
+  {default_variations, SUBSET_FONT_BASE_PATH "RobotoFlex-Variable.ttf"},
+  {default_variations, SUBSET_FONT_BASE_PATH "SourceSansPro-Regular.otf"},
+  {default_variations, SUBSET_FONT_BASE_PATH "AdobeVFPrototype.otf"},
+  {default_variations, SUBSET_FONT_BASE_PATH "SourceSerifVariable-Roman.ttf"},
+  {nullptr,            SUBSET_FONT_BASE_PATH "Comfortaa-Regular-new.ttf"},
+  {nullptr,            SUBSET_FONT_BASE_PATH "NotoNastaliqUrdu-Regular.ttf"},
+  {nullptr,            SUBSET_FONT_BASE_PATH "NotoSerifMyanmar-Regular.otf"},
 };
 
 static test_input_t *tests = default_tests;
 static unsigned num_tests = sizeof (default_tests) / sizeof (default_tests[0]);
 
-enum backend_t { HARFBUZZ, FREETYPE, CORETEXT };
-
 enum operation_t
 {
   nominal_glyphs,
   glyph_h_advances,
+  glyph_v_advances,
+  glyph_v_origins,
   glyph_extents,
   draw_glyph,
   paint_glyph,
@@ -81,7 +91,8 @@ _draw_funcs_create (void)
 }
 
 static void BM_Font (benchmark::State &state,
-		     bool is_var, backend_t backend, operation_t operation,
+		     const hb_variation_t *variations, const char * backend,
+		     operation_t operation,
 		     const test_input_t &test_input)
 {
   hb_font_t *font;
@@ -90,33 +101,26 @@ static void BM_Font (benchmark::State &state,
     hb_face_t *face = hb_benchmark_face_create_from_file_or_fail (test_input.font_path, 0);
     assert (face);
     num_glyphs = hb_face_get_glyph_count (face);
+    if (max_num_glyphs && num_glyphs > max_num_glyphs)
+      num_glyphs = max_num_glyphs;
     font = hb_font_create (face);
     hb_face_destroy (face);
   }
 
-  if (is_var)
+  if (variations)
   {
-    hb_variation_t wght = {HB_TAG ('w','g','h','t'), 500};
-    hb_font_set_variations (font, &wght, 1);
+    unsigned count = 0;
+    for (const hb_variation_t *v = variations; v->tag != HB_TAG_NONE; v++)
+      count++;
+    hb_font_set_variations (font, variations, count);
   }
 
-  switch (backend)
+  bool ret = hb_font_set_funcs_using (font, backend);
+  if (!ret)
   {
-    case HARFBUZZ:
-      hb_ot_font_set_funcs (font);
-      break;
-
-    case FREETYPE:
-#ifdef HAVE_FREETYPE
-      hb_ft_font_set_funcs (font);
-#endif
-      break;
-
-    case CORETEXT:
-#ifdef HAVE_CORETEXT
-      hb_coretext_font_set_funcs (font);
-#endif
-      break;
+    state.SkipWithError("Backend failed to initialize for font.");
+    hb_font_destroy (font);
+    return;
   }
 
   switch (operation)
@@ -164,6 +168,45 @@ static void BM_Font (benchmark::State &state,
       free (glyphs);
       break;
     }
+    case glyph_v_advances:
+    {
+      hb_codepoint_t *glyphs = (hb_codepoint_t *) calloc (num_glyphs, sizeof (hb_codepoint_t));
+      hb_position_t *advances = (hb_position_t *) calloc (num_glyphs, sizeof (hb_codepoint_t));
+
+      for (unsigned g = 0; g < num_glyphs; g++)
+        glyphs[g] = g;
+
+      for (auto _ : state)
+	hb_font_get_glyph_v_advances (font,
+				      num_glyphs,
+				      glyphs, sizeof (*glyphs),
+				      advances, sizeof (*advances));
+
+      free (advances);
+      free (glyphs);
+      break;
+    }
+    case glyph_v_origins:
+    {
+      hb_codepoint_t *glyphs = (hb_codepoint_t *) calloc (num_glyphs, sizeof (hb_codepoint_t));
+      hb_position_t *origins_x = (hb_position_t *) calloc (num_glyphs, sizeof (hb_codepoint_t));
+      hb_position_t *origins_y = (hb_position_t *) calloc (num_glyphs, sizeof (hb_codepoint_t));
+
+      for (unsigned g = 0; g < num_glyphs; g++)
+        glyphs[g] = g;
+
+      for (auto _ : state)
+        hb_font_get_glyph_v_origins (font,
+				      num_glyphs,
+				      glyphs, sizeof (*glyphs),
+				      origins_x, sizeof (*origins_x),
+				      origins_y, sizeof (*origins_y));
+
+      free (origins_y);
+      free (origins_x);
+      free (glyphs);
+      break;
+    }
     case glyph_extents:
     {
       hb_glyph_extents_t extents;
@@ -204,27 +247,16 @@ static void BM_Font (benchmark::State &state,
 	hb_font_t *font = hb_font_create (face);
 	hb_face_destroy (face);
 
-	switch (backend)
+	bool ret = hb_font_set_funcs_using (font, backend);
+	if (!ret)
 	{
-	  case HARFBUZZ:
-	    hb_ot_font_set_funcs (font);
-	    break;
-
-	  case FREETYPE:
-#ifdef HAVE_FREETYPE
-	    hb_ft_font_set_funcs (font);
-#endif
-	    break;
-
-	  case CORETEXT:
-#ifdef HAVE_CORETEXT
-	    hb_coretext_font_set_funcs (font);
-#endif
-	    break;
+	  state.SkipWithError("Backend failed to initialize for font.");
+	  hb_font_destroy (font);
+	  return;
 	}
 
 	hb_buffer_t *buffer = hb_buffer_create ();
-	hb_buffer_add_utf8 (buffer, " ", -1, 0, -1);
+	hb_buffer_add_utf8 (buffer, text, -1, 0, -1);
 	hb_buffer_guess_segment_properties (buffer);
 
 	hb_shape (font, buffer, nullptr, 0);
@@ -240,9 +272,8 @@ static void BM_Font (benchmark::State &state,
   hb_font_destroy (font);
 }
 
-static void test_backend (backend_t backend,
-			  const char *backend_name,
-			  bool variable,
+static void test_backend (const char *backend,
+			  const hb_variation_t *variations,
 			  operation_t op,
 			  const char *op_name,
 			  benchmark::TimeUnit time_unit,
@@ -253,11 +284,11 @@ static void test_backend (backend_t backend,
   strcat (name, "/");
   const char *p = strrchr (test_input.font_path, '/');
   strcat (name, p ? p + 1 : test_input.font_path);
-  strcat (name, variable ? "/var" : "");
+  strcat (name, variations ? "/var" : "");
   strcat (name, "/");
-  strcat (name, backend_name);
+  strcat (name, backend);
 
-  benchmark::RegisterBenchmark (name, BM_Font, variable, backend, op, test_input)
+  benchmark::RegisterBenchmark (name, BM_Font, variations, backend, op, test_input)
    ->Unit(time_unit);
 }
 
@@ -265,43 +296,134 @@ static void test_operation (operation_t op,
 			    const char *op_name,
 			    benchmark::TimeUnit time_unit)
 {
+  const char **supported_backends = hb_font_list_funcs ();
   for (unsigned i = 0; i < num_tests; i++)
   {
     auto& test_input = tests[i];
-    for (int variable = 0; variable < int (test_input.is_variable) + 1; variable++)
+    for (int variable = 0; variable < int (test_input.variations != nullptr) + 1; variable++)
     {
-      bool is_var = (bool) variable;
-
-      test_backend (HARFBUZZ, "hb", is_var, op, op_name, time_unit, test_input);
-#ifdef HAVE_FREETYPE
-      test_backend (FREETYPE, "ft", is_var, op, op_name, time_unit, test_input);
-#endif
-#ifdef HAVE_CORETEXT
-      test_backend (CORETEXT, "coretext", is_var, op, op_name, time_unit, test_input);
-#endif
+      const hb_variation_t *variations = variable ? test_input.variations : nullptr;
+      for (const char **backend = supported_backends; *backend; backend++)
+	test_backend (*backend, variations, op, op_name, time_unit, test_input);
     }
   }
 }
 
+static const char *font_file = nullptr;
+static const char *variations_str = nullptr;
+
+static GOptionEntry entries[] =
+{
+  {"font-file", 0, 0, G_OPTION_ARG_STRING, &font_file, "Font file-path to benchmark", "FONTFILE"},
+  {"variations", 0, 0, G_OPTION_ARG_STRING, &variations_str, "Variations to apply", "VAR"},
+  {"max-glyphs", 0, 0, G_OPTION_ARG_INT, &max_num_glyphs, "Maximum number of glyphs to process", "NUM"},
+  {"text", 0, 0, G_OPTION_ARG_STRING, &text, "Text to use for load_face_and_shape test", "TEXT"},
+  {nullptr}
+};
+
+static void print_usage (const char *prgname)
+{
+  g_print ("Usage: %s [OPTIONS] [FONTFILE]\n", prgname);
+}
+
 int main(int argc, char** argv)
 {
-  benchmark::Initialize(&argc, argv);
+  const char *prgname = g_path_get_basename (argv[0]);
 
-  if (argc > 1)
+  GOptionContext *context = g_option_context_new ("");
+  g_option_context_set_summary (context, "Benchmark font operations");
+  g_option_context_set_description (context, "Benchmark Options:");
+  g_option_context_add_main_entries (context, entries, nullptr);
+
+  bool show_help = false;
+  for (int i = 1; i < argc; i++)
   {
-    num_tests = argc - 1;
-    tests = (test_input_t *) calloc (num_tests, sizeof (test_input_t));
-    for (unsigned i = 0; i < num_tests; i++)
+    if (strcmp(argv[i], "--help") == 0 || strcmp(argv[i], "-h") == 0)
     {
-      tests[i].is_variable = true;
-      tests[i].font_path = argv[i + 1];
+      argv[i] = (char *) "--help"; // Ensure it is recognized by both google-benchmark
+      show_help = true;
+      break;
     }
   }
+  if (show_help)
+  {
+    print_usage (prgname);
+
+    gchar *help_text = g_option_context_get_help (context, false, nullptr);
+    help_text = strstr (help_text, "\n\n");
+    g_print ("%s", help_text);
+
+    benchmark::Initialize(&argc, argv); // This shows the help for google-benchmark
+  }
+
+  benchmark::Initialize(&argc, argv);
+  g_option_context_parse (context, &argc, &argv, nullptr);
+  g_option_context_free (context);
+
+  argc--;
+  argv++;
+  if (!font_file && argc)
+  {
+    font_file = *argv;
+    argc--;
+    argv++;
+  }
+
+  if (argc)
+  {
+    g_printerr ("Unexpected arguments: ");
+    for (int i = 0; i < argc; i++)
+      g_printerr ("%s ", argv[i]);
+    g_printerr ("\n\n");
+    print_usage (prgname);
+    return 1;
+  }
+
+  if (font_file)
+  {
+    unsigned num_variations = 0;
+    if (variations_str)
+    {
+      const char *p = variations_str;
+      while (p && *p)
+      {
+	num_variations++;
+	p = strchr (p, ',');
+	if (p) p++;
+      }
+    }
+    hb_variation_t *variations = variations_str ? (hb_variation_t *) calloc (num_variations, sizeof (hb_variation_t)) : nullptr;
+    if (variations_str)
+    {
+      const char *p = variations_str;
+      for (unsigned i = 0; i < num_variations; i++)
+      {
+	const char *end = strchr (p, ',');
+	if (end)
+	{
+	  hb_variation_from_string (p, end - p, &variations[i]);
+	  p = end + 1;
+	}
+	else
+	{
+	  hb_variation_from_string (p, -1, &variations[i]);
+	  p = nullptr;
+	}
+      }
+    }
+
+    num_tests = 1;
+    tests = (test_input_t *) calloc (num_tests, sizeof (test_input_t));
+    tests[0].variations = variations_str ? variations : default_variations;
+    tests[0].font_path = font_file;
+  }
 
 #define TEST_OPERATION(op, time_unit) test_operation (op, #op, time_unit)
 
   TEST_OPERATION (nominal_glyphs, benchmark::kMicrosecond);
   TEST_OPERATION (glyph_h_advances, benchmark::kMicrosecond);
+  TEST_OPERATION (glyph_v_advances, benchmark::kMicrosecond);
+  TEST_OPERATION (glyph_v_origins, benchmark::kMicrosecond);
   TEST_OPERATION (glyph_extents, benchmark::kMicrosecond);
   TEST_OPERATION (draw_glyph, benchmark::kMillisecond);
   TEST_OPERATION (paint_glyph, benchmark::kMillisecond);
@@ -313,5 +435,10 @@ int main(int argc, char** argv)
   benchmark::Shutdown();
 
   if (tests != default_tests)
+  {
+    if (tests[0].variations != default_variations)
+      free (tests[0].variations);
     free (tests);
+  }
 }
+
