@@ -14,11 +14,12 @@ struct _draw_data_t
   float path_last_y;
 };
 
+#include <cstdio>
 static void
-_move_to (hb_draw_funcs_t *dfuncs, void *draw_data_,
+_move_to (hb_draw_funcs_t *dfuncs HB_UNUSED, void *draw_data_,
 	  hb_draw_state_t *st,
 	  float to_x, float to_y,
-	  void *user_data)
+	  void *user_data HB_UNUSED)
 {
   _draw_data_t *draw_data = (_draw_data_t *) draw_data_;
   assert (!st->path_open);
@@ -27,10 +28,10 @@ _move_to (hb_draw_funcs_t *dfuncs, void *draw_data_,
 }
 
 static void
-_line_to (hb_draw_funcs_t *dfuncs, void *draw_data_,
+_line_to (hb_draw_funcs_t *dfuncs HB_UNUSED, void *draw_data_,
 	  hb_draw_state_t *st,
 	  float to_x, float to_y,
-	  void *user_data)
+	  void *user_data HB_UNUSED)
 {
   _draw_data_t *draw_data = (_draw_data_t *) draw_data_;
   assert (st->path_open);
@@ -40,11 +41,11 @@ _line_to (hb_draw_funcs_t *dfuncs, void *draw_data_,
 }
 
 static void
-_quadratic_to (hb_draw_funcs_t *dfuncs, void *draw_data_,
+_quadratic_to (hb_draw_funcs_t *dfuncs HB_UNUSED, void *draw_data_,
 	       hb_draw_state_t *st,
-	       float control_x, float control_y,
+	       float control_x HB_UNUSED, float control_y HB_UNUSED,
 	       float to_x, float to_y,
-	       void *user_data)
+	       void *user_data HB_UNUSED)
 {
   _draw_data_t *draw_data = (_draw_data_t *) draw_data_;
   assert (st->path_open);
@@ -54,12 +55,12 @@ _quadratic_to (hb_draw_funcs_t *dfuncs, void *draw_data_,
 }
 
 static void
-_cubic_to (hb_draw_funcs_t *dfuncs, void *draw_data_,
+_cubic_to (hb_draw_funcs_t *dfuncs HB_UNUSED, void *draw_data_,
 	   hb_draw_state_t *st,
-	   float control1_x, float control1_y,
-	   float control2_x, float control2_y,
+	   float control1_x HB_UNUSED, float control1_y HB_UNUSED,
+	   float control2_x HB_UNUSED, float control2_y HB_UNUSED,
 	   float to_x, float to_y,
-	   void *user_data)
+	   void *user_data HB_UNUSED)
 {
   _draw_data_t *draw_data = (_draw_data_t *) draw_data_;
   assert (st->path_open);
@@ -69,9 +70,9 @@ _cubic_to (hb_draw_funcs_t *dfuncs, void *draw_data_,
 }
 
 static void
-_close_path (hb_draw_funcs_t *dfuncs, void *draw_data_,
+_close_path (hb_draw_funcs_t *dfuncs HB_UNUSED, void *draw_data_,
 	     hb_draw_state_t *st,
-	     void *user_data)
+	     void *user_data HB_UNUSED)
 {
   _draw_data_t *draw_data = (_draw_data_t *) draw_data_;
   assert (st->path_open && draw_data->path_len != 0);
@@ -118,7 +119,7 @@ static void misc_calls_for_gid (hb_face_t *face, hb_font_t *font, hb_set_t *set,
 
 extern "C" int LLVMFuzzerTestOneInput (const uint8_t *data, size_t size)
 {
-  alloc_state = size; /* see src/failing-alloc.c */
+  alloc_state = _fuzzing_alloc_state (data, size);
 
   hb_blob_t *blob = hb_blob_create ((const char *) data, size,
 				    HB_MEMORY_MODE_READONLY, nullptr, nullptr);
@@ -150,7 +151,7 @@ extern "C" int LLVMFuzzerTestOneInput (const uint8_t *data, size_t size)
   hb_set_t *set = hb_set_create ();
   for (unsigned gid = 0; gid < glyph_count; ++gid)
   {
-    hb_font_get_glyph_shape (font, gid, funcs, &draw_data);
+    hb_font_draw_glyph (font, gid, funcs, &draw_data);
 
     /* Glyph extents also may practices the similar path, call it now that is related */
     hb_glyph_extents_t extents;
